# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - EventAggregatorSummary Action

    @copyright: 2008, 2009 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2005-2008 MoinMoin:ThomasWaldmann,
                2007 MoinMoin:ReimarBauer.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin import config
import EventAggregatorSupport

Dependencies = ['pages']

# Action class and supporting functions.

class EventAggregatorSummary(ActionBase):

    "A summary dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request

        category_list = []

        for category_name, category_pagename in \
            EventAggregatorSupport.getCategoryMapping(
                EventAggregatorSupport.getCategories(request),
                request):

            category_list.append('<option value="%s">%s</option>' % (category_pagename, category_name))

        d = {
            "buttons_html" : buttons_html,
            "category_label" : _("Categories"),
            "category_list" : "\n".join(category_list),
            "start_label" : _("Start year and month"),
            "start_default" : "",
            "end_label" : _("End year and month"),
            "end_default" : "",
            }

        return '''
<table>
    <tr>
        <td class="label"><label>%(category_label)s</label></td>
        <td class="content">
            <select multiple="multiple" name="category">
                %(category_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(start_label)s</label></td>
        <td class="content">
            <input name="start" type="text" value="%(start_default)s" size="12" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(end_label)s</label></td>
        <td class="content">
            <input name="end" type="text" value="%(end_default)s" size="12" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the iCalendar resource."

        _ = self._

        # If no category names exist in the request, an error message is
        # returned.

        category_names = self.request.form.get("category", [])

        if not category_names:
            return 0, _("No categories specified.")

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        """

        pass

def write_resource(request):

    """
    For the given 'request', write an iCalendar summary of the event data found
    in the categories specified via the "category" request parameter, using the
    "start" and "end" parameters (if specified). Multiple "category" parameters
    can be specified.
    """

    category_names = request.form.get("category", [])

    # Otherwise, produce an iCalendar resource.

    calendar_start = EventAggregatorSupport.getFormMonth(request, None, "start")
    calendar_end = EventAggregatorSupport.getFormMonth(request, None, "end")

    events, shown_events, all_shown_events, earliest, latest = \
        EventAggregatorSupport.getEvents(request, category_names, calendar_start, calendar_end)

    # Output iCalendar data...

    request.emit_http_headers(["Content-Type: text/calendar; charset=%s" % config.charset])

    request.write("BEGIN:VCALENDAR\r\n")
    request.write("PRODID:-//MoinMoin//EventAggregatorSummary\r\n")
    request.write("VERSION:2.0\r\n")

    for event_page, event_details in all_shown_events:

        event_summary = EventAggregatorSupport.getEventSummary(event_page, event_details)

        # Output the event details.

        request.write("BEGIN:VEVENT\r\n")
        request.write("SUMMARY:%s\r\n" % event_summary)
        request.write("UID:%s\r\n" % request.getQualifiedURL(event_page.url(request)))
        request.write("URL:%s\r\n" % request.getQualifiedURL(event_page.url(request)))
        request.write("DTSTART;VALUE=DATE:%04d%02d%02d\r\n" % event_details["start"])
        request.write("DTEND;VALUE=DATE:%04d%02d%02d\r\n" % EventAggregatorSupport.nextdate(event_details["end"]))
        if event_details.has_key("topics"):
            request.write("CATEGORIES:%s\r\n" % ",".join(event_details["topics"]))
        request.write("END:VEVENT\r\n")

    request.write("END:VCALENDAR\r\n")

# Action function.

def execute(pagename, request):
    EventAggregatorSummary(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
